#Import of several modules

import math
import time
import numpy as np
from scipy import integrate



#================================= Use custom forces ==================================

def custom1 (z,v,t,f0,k,Q,Rt,a0,E,E2,H,angle,sahe,mu,m,D,F0,zc,eta,sigmas,sigmat,landadeb,epsilonr,alfa,nc):
    if (z + zc) < a0:
        force = V1
    else:
        force = V2 
    return force

def custom2 (z,v,t,f0,k,Q,Rt,a0,E,E2,H,angle,sahe,mu,m,D,F0,zc,eta,sigmas,sigmat,landadeb,epsilonr,alfa,nc):
    if (z + zc) < a0:
        force = 0.3
    else:
        force = 0.3
    return force




#================================= Long range forces ==================================

def vdw (z,v,t,f0,k,Q,Rt,a0,E,E2,H,angle,sahe,mu,m,D,F0,zc,eta,sigmas,sigmat,landadeb,epsilonr,alfa,nc):
    if (z + zc) < a0:
        force = 0
    else:
        force = -H*Rt/(6*(z + zc)*(z + zc)) 
    return force

def dlvo (z,v,t,f0,k,Q,Rt,a0,E,E2,H,angle,sahe,mu,m,D,F0,zc,eta,sigmas,sigmat,landadeb,epsilonr,alfa,nc):
    epsilon = 8.854187*10**-12
    if (z + zc) < a0:
        force = 4*math.pi*Rt/(epsilon*epsilonr)*sigmat*sigmas*landadeb*np.exp(-(a0)/landadeb)
    else:
        force = -H*Rt/(6*(z + zc)*(z + zc)) + 4*math.pi*Rt/(epsilon*epsilonr)*sigmat*sigmas*landadeb*np.exp(-(z + zc)/landadeb)

    return force


#================================= Short range forces ==================================


def hertz (z,v,t,f0,k,Q,Rt,a0,E,E2,H,angle,sahe,mu,m,D,F0,zc,eta,sigmas,sigmat,landadeb,epsilonr,alfa,nc):
    if (z + zc) < a0:
        force = 4.0/3.0*E*math.sqrt(Rt)*math.pow(-(z + zc - a0),3.0/2.0)
    else:
        force = 0

    return force

def dmt (z,v,t,f0,k,Q,Rt,a0,E,E2,H,angle,sahe,mu,m,D,F0,zc,eta,sigmas,sigmat,landadeb,epsilonr,alfa,nc):
    if (z + zc) < a0:
        force = 4.0/3.0*E*math.sqrt(Rt)*math.pow(-(z + zc - a0),3.0/2.0) -H*Rt/(6*a0*a0)
    else:
        force = 0

    return force   

def tatara (z,v,t,f0,k,Q,Rt,a0,E,E2,H,angle,sahe,mu,m,D,F0,zc,eta,sigmas,sigmat,landadeb,epsilonr,alfa,nc):
    if ((z + zc) < a0) and nc > 0:
        force = alfa*math.pow(-(z + zc - a0),1.5)/math.pow(2,1.5) + 3*alfa**2*(z + zc - a0)**2/(8*nc) + 15*alfa**3/(math.pow(2,11/2.0)*nc**2)*math.pow(-(z + zc - a0),2.5) -H*Rt/(6*a0*a0)
    else:
        force = 0

    return force   


def becc (z,v,t,f0,k,Q,Rt,a0,E,E2,H,angle,sahe,mu,m,D,F0,zc,eta,sigmas,sigmat,landadeb,epsilonr,alfa,nc):
    if (z + zc) < a0:
        force = 8*E*math.tan(angle)*(-z - zc + a0)**2/(3*math.pi)*(1+1.17795*2*math.tan(angle)*(-z - zc + a0)/sahe/math.pi**2 + 16*(1.7795*math.tan(angle)*(-z - zc + a0)/sahe)**2)
    else:
        force = 0 

    return force   

def behc (z,v,t,f0,k,Q,Rt,a0,E,E2,H,angle,sahe,mu,m,D,F0,zc,eta,sigmas,sigmat,landadeb,epsilonr,alfa,nc):
    if (z + zc) < a0:
        alfa0 = -0.374*(3-2*mu)/(1-mu)
        beta0 = -0.056*(5-2*mu)/(1-mu)
        x = np.sqrt(Rt*(-z - zc + a0))/sahe
        force = 4*E*np.sqrt(Rt)*np.power((-z - zc + a0),1.5)/(3*(1-mu**2))*(1 - 2*alfa0*x/np.pi + 4*alfa**2*x**2/(np.pi**2) - 8*(alfa0**3 + 4*np.pi**2*beta0/15.)*x**3/(np.pi**3) + 16*alfa0*(alfa0**3 + 3*np.pi**2*beta0/5.)*x**4/(np.pi**4))
    else:
        force = 0 

    return force   


#================================= Viscosity forces ==================================

def visco (z,v,t,f0,k,Q,Rt,a0,E,E2,H,angle,sahe,mu,m,D,F0,zc,eta,sigmas,sigmat,landadeb,epsilonr,alfa,nc):
    if (z + zc) < a0:
        force = -eta*math.sqrt(Rt*(-z - zc + a0))*v
    else:
        force = 0

    return force

def sls (z,v,t,f0,k,Q,Rt,a0,E,E2,H,angle,sahe,mu,m,D,F0,zc,eta,sigmas,sigmat,landadeb,epsilonr,alfa,nc):
    if (z + zc) < a0:
        force = math.sqrt(Rt*(-z - zc + a0))*(E*(-z - zc + a0) - eta*delta_f/E2 - eta*(E+E2)/E2*v)
    else:
        force = 0

    return force


#-----------------------------------jkr-------------------------------------------------


forcepri = np.arange(-1,200,0.1)
identipri = 3*np.power(forcepri+2+2*np.sqrt(forcepri+1),2.0/3.0) -4*np.power(forcepri + 2 + 2*np.sqrt(forcepri+1),1/6.0)


def jkr(z,v,t,f0,k,Q,Rt,a0,E,E2,H,angle,sahe,mu,m,D,F0,zc,eta,sigmas,sigmat,landadeb,epsilonr,alfa,nc):
    ident_a = np.power(Rt*H**2*9/(3*E**2*64*36*a0**4),1/3.0)
    force_a = 9*Rt*H/(8*6*a0**2)
    if (z + zc) < a0:
        force = np.interp((-z - zc + ident_a)/ident_a,identipri,forcepri)*force_a
    elif (z + zc) < ident_a and v > 0:
        force = np.interp((-z - zc + ident_a)/ident_a,identipri,forcepri)*force_a
    else:
        force = -H*Rt/(6*(z + zc)*(z + zc))

    return force

#================================= Driven forces ==================================

def driven (F0,f0,t):
    if f0.size == 1:
        force = F0*math.cos(f0*2*math.pi*t)
    else:
        force = F0[1]*math.cos(f0[1]*2*math.pi*t) + F0[2]*math.cos(f0[2]*2*math.pi*t)

    return force



f_vector = [dlvo,vdw,hertz,dmt,tatara,visco,becc,behc,sls,jkr,custom1,custom2]

t1 = 0.0
t2 = 0.0
f1 = 0.0
f2 = 0.0
delta_f = 0.0
V1 = 0.0
V2 = 0.0
V3 = 0.0
V4 = 0.0

def custom_reset(custom_for1,custom_for2,custom_for3,custom_for4):
    global V1
    global V2
    global V3
    global V4
    V1 = custom_for1
    V2 = custom_for2
    V3 = custom_for3
    V4 = custom_for4


def sls_reset():
    global f2
    global f1
    global t2
    global t1
    global delta_f
    t1 = 0.0
    t2 = 0.0
    f1 = 0.0
    f2 = 0.0
    delta_f = 0.0

def sumatory(option_mask,z,v,t,f0,k,Q,Rt,a0,E,E2,H,H2,angle,sahe,mu,m,D,F0,zc,eta,sigmas,sigmat,landadeb,epsilonr,alfa,nc):
    if H2 > 0 and v > 0:
        H = H2
    force = sum([f(z,v,t,f0,k,Q,Rt,a0,E,E2,H,angle,sahe,mu,m,D,F0,zc,eta,sigmas,sigmat,landadeb,epsilonr,alfa,nc) for f,mask in zip(f_vector,option_mask) if mask])
    if option_mask[-1] and t2 != t:
        global f2
        global f1
        global t2
        global t1
        global delta_f
        f1 = f2
        f2 = force
        t1 = t2
        t2 = t
        delta_f = (f2 - f1)/(t2-t1)

    force = force + driven(F0,f0,t)

    return force
